<?php

/**
 * Kopano small business class.
 *
 * @category   apps
 * @package    kopano-basic
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2015-2016 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/kopano_basic/
 */

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\kopano_basic;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('base');
clearos_load_language('kopano');
clearos_load_language('kopano_basic');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\Configuration_File as Configuration_File;
use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Engine as Engine;
use \clearos\apps\base\Shell as Shell;
use \clearos\apps\flexshare\Flexshare as Flexshare;
use \clearos\apps\tasks\Cron as Cron;

clearos_load_library('base/Configuration_File');
clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Engine');
clearos_load_library('base/Shell');
clearos_load_library('flexshare/Flexshare');
clearos_load_library('tasks/Cron');

// Exceptions
//-----------

use \clearos\apps\base\Engine_Exception as Engine_Exception;
use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Engine_Exception');
clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Kopano small business class.
 *
 * @category   apps
 * @package    kopano-basic
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2015-2016 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/kopano_basic/
 */

class Kopano_Basic extends Engine
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////

    const FILE_CONFIG = '/etc/clearos/kopano.conf';
    const PATH_BACKUP = '/var/clearos/kopano_basic/backup';
    const COMMAND_KOPANO_BACKUP = '/usr/sbin/kopano-backup';
    const CRON_DAILY = '00 03 * * *  root /usr/sbin/clearos-kopano-backup';
    const CRON_WEEKLY = '00 03 * * 0  root /usr/sbin/clearos-kopano-backup';
    const CRON_MONTHLY = '00 03 1 * *  root /usr/sbin/clearos-kopano-backup';

    ///////////////////////////////////////////////////////////////////////////////
    // V A R I A B L E S
    ///////////////////////////////////////////////////////////////////////////////

    protected $is_loaded = FALSE;
    protected $config = array();

    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Kopano licensed constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);
    }

    /**
     * Run a backup.
     *
     * @return void
     * @throws Engine_Exception
     */

    public function backup()
    {
        clearos_profile(__METHOD__, __LINE__);
        $folder = new Folder($this->get_backup_destination_folder(), TRUE);
        if (!$folder->exists())
            $folder->create('root', 'root', 700);
        $args = '-O "' . $this->get_backup_destination_folder() . '"';
        $shell = new Shell();
        try {
            $exitcode = $shell->execute(self::COMMAND_KOPANO_BACKUP, $args, TRUE);
        } catch (Exception $e) {
            clearos_log('kopano-basic-api', 'Kopano SMB Backup error: ' . clearos_exception_message($e));
        }
    }

    /**
     * Set a backup schedule.
     *
     * @param int $frequency frequency of backup
     *
     * @return void
     * @throws Engine_Exception, Validation_Exception
     */

    public function set_backup_frequency($frequency)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_backup_frequency($frequency));

        $this->_set_parameter('backup_frequency', $frequency);
        $app = 'app-kopano-basic-backup';
    
        $cron = new Cron();
        if ($cron->exists_configlet($app))
            $cron->delete_configlet($app);
        switch((int)$frequency) {
            case -1:
                return;
            case 1:
                $cron_entry = self::CRON_DAILY; 
                break;
            case 2:
                $cron_entry = self::CRON_WEEKLY; 
                break;
            case 3:
                $cron_entry = self::CRON_MONTHLY; 
                break;
            default:
                return;
        }
        $cron->add_configlet($app, $cron_entry);
    }

    /**
     * Get backup schedule.
     *
     * @return int
     * @throws Engine_Exception, Validation_Exception
     */

    public function get_backup_frequency()
    {
        clearos_profile(__METHOD__, __LINE__);

        if (!$this->is_loaded)
            $this->_load_config();

        if (isset($this->config['backup_frequency']))
            return $this->config['backup_frequency'];

        return -1;
    }

    /**
     * Set a backup destination folder.
     *
     * @param int $destination destination of backup
     *
     * @return void
     * @throws Engine_Exception, Validation_Exception
     */

    public function set_backup_destination_folder($destination)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_backup_destination($destination));

        $this->_set_parameter('backup_destination', $destination);
    }

    /**
     * Get backup destination folder.
     *
     * @return string
     * @throws Engine_Exception, Validation_Exception
     */

    public function get_backup_destination_folder()
    {
        clearos_profile(__METHOD__, __LINE__);

        if (!$this->is_loaded)
            $this->_load_config();

        if (isset($this->config['backup_destination']))
            return $this->config['backup_destination'];

        return self::PATH_BACKUP;
    }

    /**
     * Get backup frequency options.
     *
     * @return array
     * @throws Engine_Exception
     */

    public function get_backup_frequency_options()
    {
        clearos_profile(__METHOD__, __LINE__);

        $options = array (
            -1 => lang('kopano_basic_no_scheduled_backups'),
            1 => lang('base_daily'),
            2 => lang('base_weekly'),
            3 => lang('base_monthly')
        );
        return $options;
    }

    /**
     * Get destination folder options.
     *
     * @return array
     * @throws Engine_Exception
     */

    public function get_backup_destination_folder_options()
    {
        clearos_profile(__METHOD__, __LINE__);

        $options = array (
            self::PATH_BACKUP => lang('kopano_basic_backup_folder_default') . ' (' . self::PATH_BACKUP . ')'
        );

        if (clearos_library_installed('flexshare/Flexshare')) {

            $flexshare = new Flexshare();
            $shares = $flexshare->get_share_summary(FALSE);
            foreach ($shares as $share)
                $options[$share['ShareDir']] = lang('flexshare_app_name') . ' - ' . $share['Name'];
        }

        // Read config file
        $destination = $this->get_backup_destination_folder();
        if (isset($destination) && !array_key_exists($destination, $options))
            $options[$destination] = lang('kopano_basic_backup_folder_custom') . ' - ' . $destination;

        return $options;
    }

    ///////////////////////////////////////////////////////////////////////////////
    // P R I V A T E   M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Loads configuration files.
     *
     * @return void
     * @throws Engine_Exception
     */

    protected function _load_config()
    {
        clearos_profile(__METHOD__, __LINE__);

        $configfile = new Configuration_File(self::FILE_CONFIG, 'match', "/(\S*)\s*=\s*\"(.*)\"/");

        try {
            $this->config = $configfile->load();
        } catch (Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }

        $this->is_loaded = TRUE;
    }

    /**
     * Generic set routine.
     *
     * @param string $key   key name
     * @param string $value value for the key
     *
     * @return  void
     * @throws Engine_Exception
     */

    function _set_parameter($key, $value)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $file = new File(self::FILE_CONFIG, TRUE);

            if (!$file->exists())
                $file->create('webconfig', 'webconfig', '0644');

            $match = $file->replace_lines("/^$key\s*=\s*/", "$key=\"$value\"\n");

            if (!$match)
                $file->add_lines("$key=\"$value\"\n");
        } catch (Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }

        $this->is_loaded = FALSE;
    }


    ///////////////////////////////////////////////////////////////////////////////
    // V A L I D A T I O N
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Validation routine for frequency.
     *
     * @param int $frequency frequency
     *
     * @return string error message if frequency is invalid
     */

    public function validate_backup_frequency($frequency)
    {
        clearos_profile(__METHOD__, __LINE__);

        $options = $this->get_backup_frequency_options();
        if (!array_key_exists($frequency, $options))
            return lang('kopano_basic_frequency_invalid');
    }

    /**
     * Validation routine for destination.
     *
     * @param int $destination destination
     *
     * @return string error message if destination is invalid
     */

    public function validate_backup_destination($destination)
    {
        clearos_profile(__METHOD__, __LINE__);

        $options = $this->get_backup_destination_folder_options();
        if (!array_key_exists($destination, $options))
            return lang('kopano_basic_destination_invalid');
    }
}
