<?php

/**
 * Kopano license class.
 *
 * @category   apps
 * @package    kopano-basic
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2016-2018 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/kopano_basic/
 */

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\kopano_basic;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('base');
clearos_load_language('kopano');
clearos_load_language('kopano_basic');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\Daemon as Daemon;
use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Shell as Shell;

clearos_load_library('base/Daemon');
clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Shell');

// Exceptions
//-----------

use \clearos\apps\base\Engine_Exception as Engine_Exception;
use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Engine_Exception');
clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Kopano license class.
 *
 * @category   apps
 * @package    kopano-basic
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2016 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/kopano_basic/
 */

class Kopano_Licensed extends Daemon
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////

    const PATH_LICENSES = '/etc/kopano/license';
    const BASE_LICENSE = 'base';
    const CAL_LICENSE = 'cal';
    const COMMAND_KOPANO_CLI = '/usr/sbin/kopano-cli';
    const FILE_SUBSCRIPTION = '/var/clearos/clearcenter/subscriptions/kopano_basic';


    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Kopano licensed constructor.
     */

    public function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);

        parent::__construct('kopano-licensed');
    }

    /**
     * Adds a license.
     *
     * @param string $serial serial number
     *
     * @return void
     * @throws Engine_Exception, Validation_Exception
     */

    public function add_license($serial)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_serial($serial));

        // Base serial numbers have 24 or more characters (need clarification).  CAL extensions have fewer characters
        if (strlen($serial) > 24) {
            $license = new File(self::PATH_LICENSES . '/' . self::BASE_LICENSE, TRUE);
            if ($license->exists())
                $license->delete();
        } else {
            // Check that a base license exists
            $existing = $this->get_licenses();
            if (!array_key_exists(self::BASE_LICENSE, $existing))
                throw new Engine_Exception(lang('kopano_base_license_required'), CLEAROS_ERROR);

            for ($i = 1; $i <= 500; $i++) {
                if (!array_key_exists(self::CAL_LICENSE . $i, $existing)) {
                    $license = new File(self::PATH_LICENSES . '/' . self::CAL_LICENSE . $i, TRUE);
                    break;
                }
            }
        }

        $license->create('root', 'root', '0644');
        $license->add_lines("$serial\n");
    }

    /**
     * Deletes a license.
     *
     * @param string $file license file
     *
     * @return void
     * @throws Engine_Exception, Validation_Exception
     */

    public function delete_license($file)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_license($file));

        $license = new File(self::PATH_LICENSES . '/' . $file, TRUE);

        if ($license->exists())
            $license->delete();
    }

    /**
     * Returns serial for given license file.
     *
     * @return array license formation
     */

    public function get_serial($file)
    {
        clearos_profile(__METHOD__, __LINE__);

        $licenses = $this->get_licenses();

        if (isset($licenses[$file]))
            return $licenses[$file];
        else
            return '';
    }

    /**
     * Returns licenses.
     *
     * Returns empty array if no licenses exist.
     *
     * @return array licenses
     */

    public function get_licenses()
    {
        clearos_profile(__METHOD__, __LINE__);

        $folder = new Folder(self::PATH_LICENSES, TRUE);

        if (! $folder->exists())
            return array();

        $license_files = $folder->get_listing();
        $licenses = array();

        foreach ($license_files as $license_file) {
            $file = new File(self::PATH_LICENSES . '/' . $license_file, TRUE);
            $licenses[$license_file] = $file->get_contents();
        }

        return $licenses;
    }

    /**
     * Updates a license.
     *
     * @param string $file   license file
     * @param string $serial serial number
     *
     * @return void
     * @throws Engine_Exception, Validation_Exception
     */

    public function update_license($file, $serial)
    {
        clearos_profile(__METHOD__, __LINE__);

        Validation_Exception::is_valid($this->validate_license($file));
        Validation_Exception::is_valid($this->validate_serial($serial));

        $license = new File(self::PATH_LICENSES . '/' . $file, TRUE);

        if ($license->exists())
            $license->delete();

        $license->create('root', 'root', '0644');
        $license->add_lines("$serial\n");
    }

    /**
     * Get license (CAL) stats.
     *
     * @return array a list of stats
     *
     * @throws Engine_Exception
     */

    public function get_cal_stats()
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $cals = array(
                'total' => 0,
                'used' => 0,
                'available' => 0
            );
            $shell = new Shell();
            $options['env'] = 'LANG=en_US';

            $exitcode = $shell->execute(self::COMMAND_KOPANO_CLI, '--user-count', TRUE, $options);
            if ($exitcode != 0)
                throw new Engine_Exception(lang('base_command_execution_failed'), CLEAROS_WARNING);
            $rows = $shell->get_output();
            foreach ($rows as $row) {
                if (preg_match('/^\s*Active\s+(\d+)$/', $row, $match)) {
                    $cals['used'] = $match[1];
                    break;
                }
            }
            // Kopano has not completed licensing engine yet...use SDN
            $file = new File(self::FILE_SUBSCRIPTION);
            if ($file->exists()) {
                if ($subscription = unserialize($file->get_contents())) {
                    if (isset($subscription->total)) {
                        $cals['total'] = $subscription->total;
                        $cals['available'] = $cals['total'] - $cals['used'];
                    }
                }
            }
            return $cals;
        } catch (Engine_Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_WARNING);
        }
    }

    ///////////////////////////////////////////////////////////////////////////////
    // V A L I D A T I O N
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Validation routine for license file.
     *
     * @param string $file license file
     *
     * @return string error message if license file is invalid
     */

    public function validate_license($file)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match('/^[A-Za-z0-9]+$/', $file))
            return lang('kopano_basic_license_invalid');
    }

    /**
     * Validation routine for serial number.
     *
     * @param string $serial serial number
     *
     * @return string error message if serial number is invalid
     */

    public function validate_serial($serial)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match('/^[A-Za-z0-9]+$/', $serial))
            return lang('kopano_basic_serial_number_invalid');
    }
}
